(* text-window.sig *)

(* Copyright (C) 2004 Dominic Gelinas
   documentation Copyright (C) 2006 Cole Hoosier

   This file is part of crypto, a cryptogram encoder/decoder.  See the
   file COPYING.txt for copying and usage restrictions *)

signature TEXT_WINDOW =
sig

structure W : WIDGET

type textwindow

(* creates a textwindow using the root, view, and args passed to it *)
val textWindow : (W.root * W.view * W.arg list) -> textwindow
(* creates a textwindow using the root, view, and args passed to it,
   but sets line-wrapping on *)
val textArea  : (W.root * W.view * W.arg list) -> textwindow
(* creates a textwindow using the root, view, and args passed to it,
   but sets line-wrapping off *)
val textField : (W.root * W.view * W.arg list) -> textwindow

(* returns the widget for a textwindow *)
val widgetOf : textwindow -> W.widget

(* returns the string currently displayed in the textwindow *)
val getText : textwindow -> string
(* sets the string to be displayed in the textwindow *)
val setText : textwindow -> string -> unit

(* returns whether the textwindow is currently active *)
val getActive : textwindow -> bool
(* sets the activity property of the textwindow *)
val setActive : textwindow -> bool -> unit

(* returns whether the textwindow is currently editable *)
val getEditable : textwindow -> bool
(* sets the editability property of the textwindow *)
val setEditable : textwindow -> bool -> unit

(* returns how many rows high the textwindow is *)
val getRows : textwindow -> int
(* sets how many rows high the textwindow should be *)
val setRows : textwindow -> int -> unit

(* returns how many columns wide the textwindow is *)
val getColumns : textwindow -> int
(* sets how many columns wide the textwindow should be *)
val setColumns : textwindow -> int -> unit

(* returns the current position of the caret in the textwindow,
   this is a 1-dimensional position with '0' being the position to the
   left of the first character on the first line *)
val getCaretPosition : textwindow -> int
(* sets the current position of the caret in the textwindow
   this is a 1-dimensional position with '0' being the position to the
   left of the first character on the first line *)
val setCaretPosition : textwindow -> int -> unit

(* returns the font currently used in the textwindow *)
val getFont : textwindow -> W.EXB.font
(* sets the font to be used in the textwindow *)
val setFont : textwindow -> W.EXB.font -> unit

(* returns the foreground color used when textwindow is active *)
val getActiveFgColor : textwindow -> W.EXB.color
(* sets the foreground color used when textwindow is active *)
val setActiveFgColor : textwindow -> W.EXB.color -> unit

(* returns the background color used when textwindow is active *)
val getActiveBgColor : textwindow -> W.EXB.color
(* sets the background color used when textwindow is active *)
val setActiveBgColor : textwindow -> W.EXB.color -> unit

(* returns the foreground color used when textwindow is inactive *)
val getInactiveFgColor : textwindow -> W.EXB.color
(* sets the foreground color used when textwindow is inactive *)
val setInactiveFgColor : textwindow -> W.EXB.color -> unit

(* returns the background color used when textwindow is inactive *)
val getInactiveBgColor : textwindow -> W.EXB.color
(* sets the background color used when textwindow is inactive *)
val setInactiveBgColor : textwindow -> W.EXB.color -> unit

end (* TEXT_WINDOW *)
